@@ -5,10 +5,9 @@
 
     A simple command line application to run flask apps.
 
-    :copyright: © 2010 by the Pallets team.
-    :license: BSD, see LICENSE for more details.
+    :copyright: 2010 Pallets
+    :license: BSD-3-Clause
 """
-
 from __future__ import print_function
 
 import ast
@@ -16,26 +15,35 @@ import inspect
 import os
 import platform
 import re
-import ssl
 import sys
 import traceback
 from functools import update_wrapper
 from operator import attrgetter
-from threading import Lock, Thread
+from threading import Lock
+from threading import Thread
 
 import click
 from werkzeug.utils import import_string
 
-from . import __version__
-from ._compat import getargspec, iteritems, reraise, text_type
+from ._compat import getargspec
+from ._compat import itervalues
+from ._compat import reraise
+from ._compat import text_type
 from .globals import current_app
-from .helpers import get_debug_flag, get_env, get_load_dotenv
+from .helpers import get_debug_flag
+from .helpers import get_env
+from .helpers import get_load_dotenv
 
 try:
     import dotenv
 except ImportError:
     dotenv = None
 
+try:
+    import ssl
+except ImportError:
+    ssl = None
+
 
 class NoAppException(click.UsageError):
     """Raised if an application cannot be found or loaded."""
@@ -48,16 +56,14 @@ def find_best_app(script_info, module):
     from . import Flask
 
     # Search for the most common names first.
-    for attr_name in ('app', 'application'):
+    for attr_name in ("app", "application"):
         app = getattr(module, attr_name, None)
 
         if isinstance(app, Flask):
             return app
 
     # Otherwise find the only object that is a Flask instance.
-    matches = [
-        v for k, v in iteritems(module.__dict__) if isinstance(v, Flask)
-    ]
+    matches = [v for v in itervalues(module.__dict__) if isinstance(v, Flask)]
 
     if len(matches) == 1:
         return matches[0]
@@ -65,11 +71,11 @@ def find_best_app(script_info, module):
         raise NoAppException(
             'Detected multiple Flask applications in module "{module}". Use '
             '"FLASK_APP={module}:name" to specify the correct '
-            'one.'.format(module=module.__name__)
+            "one.".format(module=module.__name__)
         )
 
     # Search for app factory functions.
-    for attr_name in ('create_app', 'make_app'):
+    for attr_name in ("create_app", "make_app"):
         app_factory = getattr(module, attr_name, None)
 
         if inspect.isfunction(app_factory):
@@ -83,18 +89,14 @@ def find_best_app(script_info, module):
                     raise
                 raise NoAppException(
                     'Detected factory "{factory}" in module "{module}", but '
-                    'could not call it without arguments. Use '
-                    '"FLASK_APP=\'{module}:{factory}(args)\'" to specify '
-                    'arguments.'.format(
-                        factory=attr_name, module=module.__name__
-                    )
+                    "could not call it without arguments. Use "
+                    "\"FLASK_APP='{module}:{factory}(args)'\" to specify "
+                    "arguments.".format(factory=attr_name, module=module.__name__)
                 )
 
     raise NoAppException(
         'Failed to find Flask application or factory in module "{module}". '
-        'Use "FLASK_APP={module}:name to specify one.'.format(
-            module=module.__name__
-        )
+        'Use "FLASK_APP={module}:name to specify one.'.format(module=module.__name__)
     )
 
 
@@ -107,7 +109,7 @@ def call_factory(script_info, app_factory, arguments=()):
     arg_names = args_spec.args
     arg_defaults = args_spec.defaults
 
-    if 'script_info' in arg_names:
+    if "script_info" in arg_names:
         return app_factory(*arguments, script_info=script_info)
     elif arguments:
         return app_factory(*arguments)
@@ -138,6 +140,8 @@ def _called_with_wrong_args(factory):
         # didn't reach the factory
         return True
     finally:
+        # explicitly delete tb as it is circular referenced
+        # https://docs.python.org/2/library/sys.html#sys.exc_info
         del tb
 
 
@@ -147,13 +151,14 @@ def find_app_by_string(script_info, module, app_name):
     ``script_info`` argument and calls the function with the appropriate
     arguments.
     """
-    from flask import Flask
-    match = re.match(r'^ *([^ ()]+) *(?:\((.*?) *,? *\))? *$', app_name)
+    from . import Flask
+
+    match = re.match(r"^ *([^ ()]+) *(?:\((.*?) *,? *\))? *$", app_name)
 
     if not match:
         raise NoAppException(
             '"{name}" is not a valid variable name or function '
-            'expression.'.format(name=app_name)
+            "expression.".format(name=app_name)
         )
 
     name, args = match.groups()
@@ -166,10 +171,10 @@ def find_app_by_string(script_info, module, app_name):
     if inspect.isfunction(attr):
         if args:
             try:
-                args = ast.literal_eval('({args},)'.format(args=args))
-            except (ValueError, SyntaxError)as e:
+                args = ast.literal_eval("({args},)".format(args=args))
+            except (ValueError, SyntaxError) as e:
                 raise NoAppException(
-                    'Could not parse the arguments in '
+                    "Could not parse the arguments in "
                     '"{app_name}".'.format(e=e, app_name=app_name)
                 )
         else:
@@ -183,7 +188,7 @@ def find_app_by_string(script_info, module, app_name):
 
             raise NoAppException(
                 '{e}\nThe factory "{app_name}" in module "{module}" could not '
-                'be called with the specified arguments.'.format(
+                "be called with the specified arguments.".format(
                     e=e, app_name=app_name, module=module.__name__
                 )
             )
@@ -194,10 +199,8 @@ def find_app_by_string(script_info, module, app_name):
         return app
 
     raise NoAppException(
-        'A valid Flask application was not obtained from '
-        '"{module}:{app_name}".'.format(
-            module=module.__name__, app_name=app_name
-        )
+        "A valid Flask application was not obtained from "
+        '"{module}:{app_name}".'.format(module=module.__name__, app_name=app_name)
     )
 
 
@@ -207,10 +210,11 @@ def prepare_import(path):
     """
     path = os.path.realpath(path)
 
-    if os.path.splitext(path)[1] == '.py':
-        path = os.path.splitext(path)[0]
+    fname, ext = os.path.splitext(path)
+    if ext == ".py":
+        path = fname
 
-    if os.path.basename(path) == '__init__':
+    if os.path.basename(path) == "__init__":
         path = os.path.dirname(path)
 
     module_name = []
@@ -220,17 +224,17 @@ def prepare_import(path):
         path, name = os.path.split(path)
         module_name.append(name)
 
-        if not os.path.exists(os.path.join(path, '__init__.py')):
+        if not os.path.exists(os.path.join(path, "__init__.py")):
             break
 
     if sys.path[0] != path:
         sys.path.insert(0, path)
 
-    return '.'.join(module_name[::-1])
+    return ".".join(module_name[::-1])
 
 
 def locate_app(script_info, module_name, app_name, raise_if_not_found=True):
-    __traceback_hide__ = True
+    __traceback_hide__ = True  # noqa: F841
 
     try:
         __import__(module_name)
@@ -240,12 +244,10 @@ def locate_app(script_info, module_name, app_name, raise_if_not_found=True):
         if sys.exc_info()[-1].tb_next:
             raise NoAppException(
                 'While importing "{name}", an ImportError was raised:'
-                '\n\n{tb}'.format(name=module_name, tb=traceback.format_exc())
+                "\n\n{tb}".format(name=module_name, tb=traceback.format_exc())
             )
         elif raise_if_not_found:
-            raise NoAppException(
-                'Could not import "{name}".'.format(name=module_name)
-            )
+            raise NoAppException('Could not import "{name}".'.format(name=module_name))
         else:
             return
 
@@ -260,27 +262,30 @@ def locate_app(script_info, module_name, app_name, raise_if_not_found=True):
 def get_version(ctx, param, value):
     if not value or ctx.resilient_parsing:
         return
+
     import werkzeug
-    message = (
-        'Python %(python)s\n'
-        'Flask %(flask)s\n'
-        'Werkzeug %(werkzeug)s'
+    from . import __version__
+
+    message = "Python %(python)s\nFlask %(flask)s\nWerkzeug %(werkzeug)s"
+    click.echo(
+        message
+        % {
+            "python": platform.python_version(),
+            "flask": __version__,
+            "werkzeug": werkzeug.__version__,
+        },
+        color=ctx.color,
     )
-    click.echo(message % {
-        'python': platform.python_version(),
-        'flask': __version__,
-        'werkzeug': werkzeug.__version__,
-    }, color=ctx.color)
     ctx.exit()
 
 
 version_option = click.Option(
-    ['--version'],
-    help='Show the flask version',
+    ["--version"],
+    help="Show the flask version",
     expose_value=False,
     callback=get_version,
     is_flag=True,
-    is_eager=True
+    is_eager=True,
 )
 
 
@@ -303,30 +308,31 @@ class DispatchingApp(object):
 
     def _load_in_background(self):
         def _load_app():
-            __traceback_hide__ = True
+            __traceback_hide__ = True  # noqa: F841
             with self._lock:
                 try:
                     self._load_unlocked()
                 except Exception:
                     self._bg_loading_exc_info = sys.exc_info()
+
         t = Thread(target=_load_app, args=())
         t.start()
 
     def _flush_bg_loading_exception(self):
-        __traceback_hide__ = True
+        __traceback_hide__ = True  # noqa: F841
         exc_info = self._bg_loading_exc_info
         if exc_info is not None:
             self._bg_loading_exc_info = None
             reraise(*exc_info)
 
     def _load_unlocked(self):
-        __traceback_hide__ = True
+        __traceback_hide__ = True  # noqa: F841
         self._app = rv = self.loader()
         self._bg_loading_exc_info = None
         return rv
 
     def __call__(self, environ, start_response):
-        __traceback_hide__ = True
+        __traceback_hide__ = True  # noqa: F841
         if self._app is not None:
             return self._app(environ, start_response)
         self._flush_bg_loading_exception()
@@ -347,10 +353,9 @@ class ScriptInfo(object):
     onwards as click object.
     """
 
-    def __init__(self, app_import_path=None, create_app=None,
-                 set_debug_flag=True):
+    def __init__(self, app_import_path=None, create_app=None, set_debug_flag=True):
         #: Optionally the import path for the Flask application.
-        self.app_import_path = app_import_path or os.environ.get('FLASK_APP')
+        self.app_import_path = app_import_path or os.environ.get("FLASK_APP")
         #: Optionally a function that is passed the script info to create
         #: the instance of the application.
         self.create_app = create_app
@@ -365,7 +370,7 @@ class ScriptInfo(object):
         this multiple times will just result in the already loaded app to
         be returned.
         """
-        __traceback_hide__ = True
+        __traceback_hide__ = True  # noqa: F841
 
         if self._loaded_app is not None:
             return self._loaded_app
@@ -376,21 +381,22 @@ class ScriptInfo(object):
             app = call_factory(self, self.create_app)
         else:
             if self.app_import_path:
-                path, name = (re.split(r':(?![\\/])', self.app_import_path, 1) + [None])[:2]
+                path, name = (
+                    re.split(r":(?![\\/])", self.app_import_path, 1) + [None]
+                )[:2]
                 import_name = prepare_import(path)
                 app = locate_app(self, import_name, name)
             else:
-                for path in ('wsgi.py', 'app.py'):
+                for path in ("wsgi.py", "app.py"):
                     import_name = prepare_import(path)
-                    app = locate_app(self, import_name, None,
-                                     raise_if_not_found=False)
+                    app = locate_app(self, import_name, None, raise_if_not_found=False)
 
                     if app:
                         break
 
         if not app:
             raise NoAppException(
-                'Could not locate a Flask application. You did not provide '
+                "Could not locate a Flask application. You did not provide "
                 'the "FLASK_APP" environment variable, and a "wsgi.py" or '
                 '"app.py" module was not found in the current directory.'
             )
@@ -413,10 +419,12 @@ def with_appcontext(f):
     to the ``app.cli`` object then they are wrapped with this function
     by default unless it's disabled.
     """
+
     @click.pass_context
     def decorator(__ctx, *args, **kwargs):
         with __ctx.ensure_object(ScriptInfo).load_app().app_context():
             return __ctx.invoke(f, *args, **kwargs)
+
     return update_wrapper(decorator, f)
 
 
@@ -433,11 +441,13 @@ class AppGroup(click.Group):
         :class:`click.Group` but it wraps callbacks in :func:`with_appcontext`
         unless it's disabled by passing ``with_appcontext=False``.
         """
-        wrap_for_ctx = kwargs.pop('with_appcontext', True)
+        wrap_for_ctx = kwargs.pop("with_appcontext", True)
+
         def decorator(f):
             if wrap_for_ctx:
                 f = with_appcontext(f)
             return click.Group.command(self, *args, **kwargs)(f)
+
         return decorator
 
     def group(self, *args, **kwargs):
@@ -445,7 +455,7 @@ class AppGroup(click.Group):
         :class:`click.Group` but it defaults the group class to
         :class:`AppGroup`.
         """
-        kwargs.setdefault('cls', AppGroup)
+        kwargs.setdefault("cls", AppGroup)
         return click.Group.group(self, *args, **kwargs)
 
 
@@ -459,7 +469,7 @@ class FlaskGroup(AppGroup):
     For information as of why this is useful see :ref:`custom-scripts`.
 
     :param add_default_commands: if this is True then the default run and
-        shell commands wil be added.
+        shell commands will be added.
     :param add_version_option: adds the ``--version`` option.
     :param create_app: an optional callback that is passed the script info and
         returns the loaded app.
@@ -474,10 +484,16 @@ class FlaskGroup(AppGroup):
         from :file:`.env` and :file:`.flaskenv` files.
     """
 
-    def __init__(self, add_default_commands=True, create_app=None,
-                 add_version_option=True, load_dotenv=True,
-                 set_debug_flag=True, **extra):
-        params = list(extra.pop('params', None) or ())
+    def __init__(
+        self,
+        add_default_commands=True,
+        create_app=None,
+        add_version_option=True,
+        load_dotenv=True,
+        set_debug_flag=True,
+        **extra
+    ):
+        params = list(extra.pop("params", None) or ())
 
         if add_version_option:
             params.append(version_option)
@@ -503,7 +519,7 @@ class FlaskGroup(AppGroup):
             self._loaded_plugin_commands = True
             return
 
-        for ep in pkg_resources.iter_entry_points('flask.commands'):
+        for ep in pkg_resources.iter_entry_points("flask.commands"):
             self.add_command(ep.load(), ep.name)
         self._loaded_plugin_commands = True
 
@@ -553,19 +569,20 @@ class FlaskGroup(AppGroup):
         # command line interface. This is detected by Flask.run to make the
         # call into a no-op. This is necessary to avoid ugly errors when the
         # script that is loaded here also attempts to start a server.
-        os.environ['FLASK_RUN_FROM_CLI'] = 'true'
+        os.environ["FLASK_RUN_FROM_CLI"] = "true"
 
         if get_load_dotenv(self.load_dotenv):
             load_dotenv()
 
-        obj = kwargs.get('obj')
+        obj = kwargs.get("obj")
 
         if obj is None:
-            obj = ScriptInfo(create_app=self.create_app,
-                             set_debug_flag=self.set_debug_flag)
+            obj = ScriptInfo(
+                create_app=self.create_app, set_debug_flag=self.set_debug_flag
+            )
 
-        kwargs['obj'] = obj
-        kwargs.setdefault('auto_envvar_prefix', 'FLASK')
+        kwargs["obj"] = obj
+        kwargs.setdefault("auto_envvar_prefix", "FLASK")
         return super(FlaskGroup, self).main(*args, **kwargs)
 
 
@@ -573,7 +590,7 @@ def _path_is_ancestor(path, other):
     """Take ``other`` and remove the length of ``path`` from it. Then join it
     to ``path``. If it is the original value, ``path`` is an ancestor of
     ``other``."""
-    return os.path.join(path, other[len(path):].lstrip(os.sep)) == other
+    return os.path.join(path, other[len(path) :].lstrip(os.sep)) == other
 
 
 def load_dotenv(path=None):
@@ -593,22 +610,34 @@ def load_dotenv(path=None):
     :param path: Load the file at this location instead of searching.
     :return: ``True`` if a file was loaded.
 
+    .. versionchanged:: 1.1.0
+        Returns ``False`` when python-dotenv is not installed, or when
+        the given path isn't a file.
+
     .. versionadded:: 1.0
     """
     if dotenv is None:
-        if path or os.path.exists('.env') or os.path.exists('.flaskenv'):
+        if path or os.path.isfile(".env") or os.path.isfile(".flaskenv"):
             click.secho(
-                ' * Tip: There are .env files present.'
+                " * Tip: There are .env or .flaskenv files present."
                 ' Do "pip install python-dotenv" to use them.',
-                fg='yellow')
-        return
+                fg="yellow",
+                err=True,
+            )
 
+        return False
+
+    # if the given path specifies the actual file then return True,
+    # else False
     if path is not None:
-        return dotenv.load_dotenv(path)
+        if os.path.isfile(path):
+            return dotenv.load_dotenv(path)
+
+        return False
 
     new_dir = None
 
-    for name in ('.env', '.flaskenv'):
+    for name in (".env", ".flaskenv"):
         path = dotenv.find_dotenv(name, usecwd=True)
 
         if not path:
@@ -629,27 +658,29 @@ def show_server_banner(env, debug, app_import_path, eager_loading):
     """Show extra startup messages the first time the server is run,
     ignoring the reloader.
     """
-    if os.environ.get('WERKZEUG_RUN_MAIN') == 'true':
+    if os.environ.get("WERKZEUG_RUN_MAIN") == "true":
         return
 
     if app_import_path is not None:
         message = ' * Serving Flask app "{0}"'.format(app_import_path)
 
         if not eager_loading:
-            message += ' (lazy loading)'
+            message += " (lazy loading)"
 
         click.echo(message)
 
-    click.echo(' * Environment: {0}'.format(env))
+    click.echo(" * Environment: {0}".format(env))
 
-    if env == 'production':
+    if env == "production":
         click.secho(
-            '   WARNING: This is a development server. '
-            'Do not use it in a production deployment.', fg='red')
-        click.secho('   Use a production WSGI server instead.', dim=True)
+            "   WARNING: This is a development server. "
+            "Do not use it in a production deployment.",
+            fg="red",
+        )
+        click.secho("   Use a production WSGI server instead.", dim=True)
 
     if debug is not None:
-        click.echo(' * Debug mode: {0}'.format('on' if debug else 'off'))
+        click.echo(" * Debug mode: {0}".format("on" if debug else "off"))
 
 
 class CertParamType(click.ParamType):
@@ -658,25 +689,31 @@ class CertParamType(click.ParamType):
     :class:`~ssl.SSLContext` object.
     """
 
-    name = 'path'
+    name = "path"
 
     def __init__(self):
-        self.path_type = click.Path(
-            exists=True, dir_okay=False, resolve_path=True)
+        self.path_type = click.Path(exists=True, dir_okay=False, resolve_path=True)
 
     def convert(self, value, param, ctx):
+        if ssl is None:
+            raise click.BadParameter(
+                'Using "--cert" requires Python to be compiled with SSL support.',
+                ctx,
+                param,
+            )
+
         try:
             return self.path_type(value, param, ctx)
         except click.BadParameter:
             value = click.STRING(value, param, ctx).lower()
 
-            if value == 'adhoc':
+            if value == "adhoc":
                 try:
-                    import OpenSSL
+                    import OpenSSL  # noqa: F401
                 except ImportError:
                     raise click.BadParameter(
-                        'Using ad-hoc certificates requires pyOpenSSL.',
-                        ctx, param)
+                        "Using ad-hoc certificates requires pyOpenSSL.", ctx, param
+                    )
 
                 return value
 
@@ -696,8 +733,8 @@ def _validate_key(ctx, param, value):
     """The ``--key`` option must be specified when ``--cert`` is a file.
     Modifies the ``cert`` param to be a ``(cert, key)`` pair if needed.
     """
-    cert = ctx.params.get('cert')
-    is_adhoc = cert == 'adhoc'
+    cert = ctx.params.get("cert")
+    is_adhoc = cert == "adhoc"
 
     if sys.version_info < (2, 7, 9):
         is_context = cert and not isinstance(cert, (text_type, bytes))
@@ -707,55 +744,87 @@ def _validate_key(ctx, param, value):
     if value is not None:
         if is_adhoc:
             raise click.BadParameter(
-                'When "--cert" is "adhoc", "--key" is not used.',
-                ctx, param)
+                'When "--cert" is "adhoc", "--key" is not used.', ctx, param
+            )
 
         if is_context:
             raise click.BadParameter(
-                'When "--cert" is an SSLContext object, "--key is not used.',
-                ctx, param)
+                'When "--cert" is an SSLContext object, "--key is not used.', ctx, param
+            )
 
         if not cert:
-            raise click.BadParameter(
-                '"--cert" must also be specified.',
-                ctx, param)
+            raise click.BadParameter('"--cert" must also be specified.', ctx, param)
 
-        ctx.params['cert'] = cert, value
+        ctx.params["cert"] = cert, value
 
     else:
         if cert and not (is_adhoc or is_context):
-            raise click.BadParameter(
-                'Required when using "--cert".',
-                ctx, param)
+            raise click.BadParameter('Required when using "--cert".', ctx, param)
 
     return value
 
 
-@click.command('run', short_help='Run a development server.')
-@click.option('--host', '-h', default='127.0.0.1',
-              help='The interface to bind to.')
-@click.option('--port', '-p', default=5000,
-              help='The port to bind to.')
-@click.option('--cert', type=CertParamType(),
-              help='Specify a certificate file to use HTTPS.')
-@click.option('--key',
-              type=click.Path(exists=True, dir_okay=False, resolve_path=True),
-              callback=_validate_key, expose_value=False,
-              help='The key file to use when specifying a certificate.')
-@click.option('--reload/--no-reload', default=None,
-              help='Enable or disable the reloader. By default the reloader '
-              'is active if debug is enabled.')
-@click.option('--debugger/--no-debugger', default=None,
-              help='Enable or disable the debugger. By default the debugger '
-              'is active if debug is enabled.')
-@click.option('--eager-loading/--lazy-loader', default=None,
-              help='Enable or disable eager loading. By default eager '
-              'loading is enabled if the reloader is disabled.')
-@click.option('--with-threads/--without-threads', default=True,
-              help='Enable or disable multithreading.')
+class SeparatedPathType(click.Path):
+    """Click option type that accepts a list of values separated by the
+    OS's path separator (``:``, ``;`` on Windows). Each value is
+    validated as a :class:`click.Path` type.
+    """
+
+    def convert(self, value, param, ctx):
+        items = self.split_envvar_value(value)
+        super_convert = super(SeparatedPathType, self).convert
+        return [super_convert(item, param, ctx) for item in items]
+
+
+@click.command("run", short_help="Run a development server.")
+@click.option("--host", "-h", default="127.0.0.1", help="The interface to bind to.")
+@click.option("--port", "-p", default=5000, help="The port to bind to.")
+@click.option(
+    "--cert", type=CertParamType(), help="Specify a certificate file to use HTTPS."
+)
+@click.option(
+    "--key",
+    type=click.Path(exists=True, dir_okay=False, resolve_path=True),
+    callback=_validate_key,
+    expose_value=False,
+    help="The key file to use when specifying a certificate.",
+)
+@click.option(
+    "--reload/--no-reload",
+    default=None,
+    help="Enable or disable the reloader. By default the reloader "
+    "is active if debug is enabled.",
+)
+@click.option(
+    "--debugger/--no-debugger",
+    default=None,
+    help="Enable or disable the debugger. By default the debugger "
+    "is active if debug is enabled.",
+)
+@click.option(
+    "--eager-loading/--lazy-loader",
+    default=None,
+    help="Enable or disable eager loading. By default eager "
+    "loading is enabled if the reloader is disabled.",
+)
+@click.option(
+    "--with-threads/--without-threads",
+    default=True,
+    help="Enable or disable multithreading.",
+)
+@click.option(
+    "--extra-files",
+    default=None,
+    type=SeparatedPathType(),
+    help=(
+        "Extra files that trigger a reload on change. Multiple paths"
+        " are separated by '{}'.".format(os.path.pathsep)
+    ),
+)
 @pass_script_info
-def run_command(info, host, port, reload, debugger, eager_loading,
-                with_threads, cert):
+def run_command(
+    info, host, port, reload, debugger, eager_loading, with_threads, cert, extra_files
+):
     """Run a local development server.
 
     This server is for development purposes only. It does not provide
@@ -779,11 +848,20 @@ def run_command(info, host, port, reload, debugger, eager_loading,
     app = DispatchingApp(info.load_app, use_eager_loading=eager_loading)
 
     from werkzeug.serving import run_simple
-    run_simple(host, port, app, use_reloader=reload, use_debugger=debugger,
-               threaded=with_threads, ssl_context=cert)
+
+    run_simple(
+        host,
+        port,
+        app,
+        use_reloader=reload,
+        use_debugger=debugger,
+        threaded=with_threads,
+        ssl_context=cert,
+        extra_files=extra_files,
+    )
 
 
-@click.command('shell', short_help='Run a shell in the app context.')
+@click.command("shell", short_help="Run a shell in the app context.")
 @with_appcontext
 def shell_command():
     """Run an interactive Python shell in the context of a given
@@ -794,9 +872,10 @@ def shell_command():
     without having to manually configure the application.
     """
     import code
-    from flask.globals import _app_ctx_stack
+    from .globals import _app_ctx_stack
+
     app = _app_ctx_stack.top.app
-    banner = 'Python %s on %s\nApp: %s [%s]\nInstance: %s' % (
+    banner = "Python %s on %s\nApp: %s [%s]\nInstance: %s" % (
         sys.version,
         sys.platform,
         app.import_name,
@@ -807,68 +886,64 @@ def shell_command():
 
     # Support the regular Python interpreter startup script if someone
     # is using it.
-    startup = os.environ.get('PYTHONSTARTUP')
+    startup = os.environ.get("PYTHONSTARTUP")
     if startup and os.path.isfile(startup):
-        with open(startup, 'r') as f:
-            eval(compile(f.read(), startup, 'exec'), ctx)
+        with open(startup, "r") as f:
+            eval(compile(f.read(), startup, "exec"), ctx)
 
     ctx.update(app.make_shell_context())
 
     code.interact(banner=banner, local=ctx)
 
 
-@click.command('routes', short_help='Show the routes for the app.')
+@click.command("routes", short_help="Show the routes for the app.")
 @click.option(
-    '--sort', '-s',
-    type=click.Choice(('endpoint', 'methods', 'rule', 'match')),
-    default='endpoint',
+    "--sort",
+    "-s",
+    type=click.Choice(("endpoint", "methods", "rule", "match")),
+    default="endpoint",
     help=(
         'Method to sort routes by. "match" is the order that Flask will match '
-        'routes when dispatching a request.'
-    )
-)
-@click.option(
-    '--all-methods',
-    is_flag=True,
-    help="Show HEAD and OPTIONS methods."
+        "routes when dispatching a request."
+    ),
 )
+@click.option("--all-methods", is_flag=True, help="Show HEAD and OPTIONS methods.")
 @with_appcontext
 def routes_command(sort, all_methods):
     """Show all registered routes with endpoints and methods."""
 
     rules = list(current_app.url_map.iter_rules())
     if not rules:
-        click.echo('No routes were registered.')
+        click.echo("No routes were registered.")
         return
 
-    ignored_methods = set(() if all_methods else ('HEAD', 'OPTIONS'))
+    ignored_methods = set(() if all_methods else ("HEAD", "OPTIONS"))
 
-    if sort in ('endpoint', 'rule'):
+    if sort in ("endpoint", "rule"):
         rules = sorted(rules, key=attrgetter(sort))
-    elif sort == 'methods':
+    elif sort == "methods":
         rules = sorted(rules, key=lambda rule: sorted(rule.methods))
 
-    rule_methods = [
-        ', '.join(sorted(rule.methods - ignored_methods)) for rule in rules
-    ]
+    rule_methods = [", ".join(sorted(rule.methods - ignored_methods)) for rule in rules]
 
-    headers = ('Endpoint', 'Methods', 'Rule')
+    headers = ("Endpoint", "Methods", "Rule")
     widths = (
         max(len(rule.endpoint) for rule in rules),
         max(len(methods) for methods in rule_methods),
         max(len(rule.rule) for rule in rules),
     )
     widths = [max(len(h), w) for h, w in zip(headers, widths)]
-    row = '{{0:<{0}}}  {{1:<{1}}}  {{2:<{2}}}'.format(*widths)
+    row = "{{0:<{0}}}  {{1:<{1}}}  {{2:<{2}}}".format(*widths)
 
     click.echo(row.format(*headers).strip())
-    click.echo(row.format(*('-' * width for width in widths)))
+    click.echo(row.format(*("-" * width for width in widths)))
 
     for rule, methods in zip(rules, rule_methods):
         click.echo(row.format(rule.endpoint, methods, rule.rule).rstrip())
 
 
-cli = FlaskGroup(help="""\
+cli = FlaskGroup(
+    help="""\
 A general utility script for Flask applications.
 
 Provides commands from Flask, extensions, and the application. Loads the
@@ -881,30 +956,15 @@ debug mode.
   {prefix}{cmd} FLASK_ENV=development
   {prefix}flask run
 """.format(
-    cmd='export' if os.name == 'posix' else 'set',
-    prefix='$ ' if os.name == 'posix' else '> '
-))
+        cmd="export" if os.name == "posix" else "set",
+        prefix="$ " if os.name == "posix" else "> ",
+    )
+)
 
 
 def main(as_module=False):
-    args = sys.argv[1:]
-
-    if as_module:
-        this_module = 'flask'
-
-        if sys.version_info < (2, 7):
-            this_module += '.cli'
-
-        name = 'python -m ' + this_module
-
-        # Python rewrites "python -m flask" to the path to the file in argv.
-        # Restore the original command so that the reloader works.
-        sys.argv = ['-m', this_module] + args
-    else:
-        name = None
-
-    cli.main(args=args, prog_name=name)
+    cli.main(prog_name="python -m flask" if as_module else None)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main(as_module=True)
